@@ -5,6 +5,8 @@
  * parsing and provide list of allowed CSS classes.
  */
 
+(function(jQuery){ // protect against jQuery being redefined
+
 function PresentationControls(wym, opts) {
     this.wym = wym;
     this.opts = opts;
@@ -30,7 +32,7 @@ function PresentationControls(wym, opts) {
     // Need to sync with presentation.py
     this.blockdefSelector = "h1,h2,h3,h4,h5,h6,p,ol,ul,blockquote,li,pre";
 
-    this.setupControls(jQuery(wym._bvox).find(".wym_area_bottom"));
+    this.setupControls(jQuery(wym._box).find(".wym_area_right"));
 }
 
 // ---- Setup and loading ----
@@ -44,14 +46,17 @@ PresentationControls.prototype.setupControls = function(container) {
     var previewBoxId = idPrefix + 'previewbox';
     var hidePreviewButtonId = idPrefix + 'hidepreviewbutton';
     var cleanHtmlButtonId = idPrefix + 'cleanhtmlbutton';
+    var editorNewTabId = idPrefix + 'editornewtab';
     var self = this;
 
     // Create elements
-    container.after(
-        "<div class=\"prescontrol\">" +
-            "<input type=\"submit\" value=\"Clean pasted HTML\" id=\"" + cleanHtmlButtonId  +  "\" />" +
-            "<input type=\"submit\" value=\"Preview\" id=\"" + previewButtonId + "\" />" +
-            "<input type=\"submit\" id=\"" + showStylesButtonId + "\" />" +
+    container.prepend(
+        "<div class=\"prescontrol wym_panel\">" +
+            "<h2>Semantic editor</h2>" +
+            "<input type=\"submit\" id=\"" + showStylesButtonId + "\" /><br/>" +
+            "<input type=\"submit\" value=\"Clean pasted HTML\" id=\"" + cleanHtmlButtonId  +  "\" /><br/>" +
+            "<input type=\"submit\" value=\"Preview\" id=\"" + previewButtonId + "\" /><br/>" +
+            "<input type=\"submit\" value=\"Edit in new tab\" id=\"" + editorNewTabId + "\" /<br/>" +
             "<div class=\"prescontrolerror\" id=\"" + idPrefix + "errorbox" + "\"></div>" +
         "</div>");
 
@@ -70,6 +75,7 @@ PresentationControls.prototype.setupControls = function(container) {
     this.hidePreviewButton = jQuery('#' + hidePreviewButtonId);
     this.showStylesButton = jQuery('#' + showStylesButtonId);
     this.cleanHtmlButton = jQuery('#' + cleanHtmlButtonId);
+    this.editorNewTabButton = jQuery('#' + editorNewTabId);
 
     this.setupCssStorage();
 
@@ -103,6 +109,11 @@ PresentationControls.prototype.setupControls = function(container) {
                                    self.cleanHtml();
                                    return false;
                                });
+    this.editorNewTabButton.click(function(event) {
+                                      var url = self.wym._box.get(0).ownerDocument.URL;
+                                      window.open(url, '_blank');
+                                      return false;
+                                  });
     jQuery(this.wym._doc)
         .bind("keyup", function(evt) {
                   // this style of binding gives docKeyup
@@ -631,10 +642,55 @@ PresentationControls.prototype.updateStyleDisplay = function(sectId) {
     var styles = this.presentationInfo[sectId];
     var self = this;
     var stylelist = jQuery.map(styles, function(s, i) {
-                                   return self.getVerboseStyleName(s.name);
-                               }).join(", ");
-    // put a non-breaking space into the style list, so it always exists                           
-    this.addCssRule("#" + sectId + ":before", 'content: "' + stylelist + '\\00a0' + '"');
+                                   return self.getPresentationInfo(s.name);
+                               });
+    // Group by category:
+    var cmpString = function(a, b) {
+        if (a == null && b == null) {
+            return 0;
+        }
+        if (a == null) {
+            return -1;
+        }
+        if (b == null) {
+            return 1;
+        }
+        if (a < b) {
+            return -1;
+        }
+        if (a > b) {
+            return 1;
+        }
+        return 0;
+    };
+    stylelist.sort(function(s1, s2) {
+                       if (s1.category == s2.category) {
+                           return cmpString(s1.verbose_name, s2.verbose_name);
+                       } else {
+                           return cmpString(s1.category, s2.category);
+                       }
+                   });
+
+    // Now build the caption.
+    var caption = [];
+    var currentCategory = null;
+    jQuery.each(stylelist, function(i, item) {
+        if (caption.length > 0) {
+            caption.push(", ");
+        }
+        if (currentCategory != item.category) {
+            // NB - due to sorting, uncategorized items always appear at
+            // start, which is what we want.
+            if (item.category != null) {
+                caption.push(item.category + ": ");
+            }
+            currentCategory = item.category;
+        }
+        caption.push(item.verbose_name);
+    });
+
+    // put a non-breaking space into the style list, so it always exists
+    this.addCssRule("#" + sectId + ":before", 'content: "' + caption.join("") + '\\00a0' + '"');
 };
 
 PresentationControls.prototype.updateAllStyleDisplay = function() {
@@ -643,20 +699,20 @@ PresentationControls.prototype.updateAllStyleDisplay = function() {
     }
 };
 
-PresentationControls.prototype.getVerboseStyleName = function(stylename) {
-    // Full style information is not stored against individual headings, only
-    // the type and name. So sometimes we need to go from one to the other.
+PresentationControls.prototype.getPresentationInfo = function(stylename) {
+    // Full PresentationInfo information is not stored against individual DOM nodes, only
+    // the prestype and name. This function gets the full info from the name.
 
     var styles = this.availableStyles;
     for (var i = 0; i < styles.length; i++) {
         if (styles[i].name == stylename) {
-            return styles[i].verbose_name;
+            return styles[i];
         }
     }
     var commands = this.commands;
     for (var i2 = 0; i2 < commands; i2++) {
         if (commands[i2].name == stylename) {
-            return commands[i2].verbose_name;
+            return commands[i2];
         }
     }
     return undefined; // shouldn't get here
@@ -727,6 +783,7 @@ PresentationControls.prototype.updateClassListItem = function(btn, style) {
             btn.removeClass("used");
         } else {
             btn.hide();
+            btn.addClass('disabled');
         }
     } else {
         if (style.prestype == "command") {
@@ -737,6 +794,7 @@ PresentationControls.prototype.updateClassListItem = function(btn, style) {
                 btn.removeClass("used");
             }
         } else {
+            btn.removeClass('disabled');
             btn.show();
             if (this.hasStyle(sectId, style)) {
                 btn.addClass("used");
@@ -769,6 +827,16 @@ PresentationControls.prototype.updateAppliedButtons = function(curContainer) {
                                             self.updateClassListItem(jQuery(this), styles[k]);
                                         });
         }
+        this.classList.find('li.class-category').each(function(k) {
+            var category = jQuery(this);
+            var total = category.find('li a').length;
+            var hidden = category.find('li a.disabled').length;
+            if ((total - hidden) > 0) {
+                category.show();
+            } else {
+                category.hide();
+            }
+        });
 
         // We also update the container list to set the class="used" for
         // consistent styling between that list and the command/class lists.
@@ -931,12 +999,11 @@ PresentationControls.prototype.showPreview = function() {
                 function(data) {
                     self.withGoodData(data,
                         function(value) {
-                            var btn = self.previewButton;
                             var box = self.previewBox;
-                            var pos = btn.offset();
+                            var pos = jQuery(self.wym._iframe).offset();
                             box.find(".content").html(value.html);
                             var height = box.outerHeight();
-                            box.css("top", pos.top - height - 20).css("left", pos.left);
+                            box.css("top", pos.top + 20).css("left", pos.left + 20);
                             box.show();
                         });
                 }, "json");
@@ -947,7 +1014,7 @@ PresentationControls.prototype.showStyles = function(show) {
     if (show) {
         jQuery(this.wym._doc).find('#presmodestyles').remove();
         jQuery(this.wym._iframe.ownerDocument).find('#presmodestyles').remove();
-        this.showStylesButton.attr('value', 'Hide styles');
+        this.showStylesButton.attr('value', 'Hide presentation layer');
         // Fix height of list of classes.
         var h = jQuery(" .wym_area_main").outerHeight() -
             jQuery(this.wym._options.containersSelector).outerHeight() -
@@ -965,7 +1032,7 @@ PresentationControls.prototype.showStyles = function(show) {
         jQuery('<style rel="stylesheet" type="text/css" id="presmodestyles">' +
                '.wym_layout_commands, .wym_classes  {display: none!important;} ' +
                '</style>').appendTo(jQuery(this.wym._iframe.ownerDocument).find("head"));
-        this.showStylesButton.attr('value', 'Show styles');
+        this.showStylesButton.attr('value', 'Show presentation layer');
     }
 };
 
@@ -976,3 +1043,6 @@ WYMeditor.editor.prototype.semantic = function(options) {
     var wym = this;
     var c = new PresentationControls(wym, options);
 };
+
+
+})(jQuery);
